% This illustrates EE with homotopy for a simple 2D dataset projecting to 1D

% Dataset
N=100; t=linspace(0,2*pi,N)'; Y=[t sin(t)];
figure(1); plot(Y(:,1),Y(:,2),'bo'); set(gca,'DataAspectRatio',[1 1 1]);

% EE model and initial parameters
X0 = 1e-50*rand(N,1);
kp=Inf; s=1; Wp = gaussaff(Y,{'k',kp},s); % kp=10
Wn = ones(N,N); % kn=70; s=Inf; Wn = gaussaff(Y,{'k',kn},s);
Wn = Wn.*sqdist(Y);
% Ensure symmetry and zero diagonal
Wp=(Wp+Wp')/2; Wn=(Wn+Wn')/2; Wp(1:N+1:N^2)=0; Wn(1:N+1:N^2)=0;

% Compute bounds for critical lambda
[ub,lb,ublist,lblist,Lp,evecLp,evalLp,Ln,evecLn,evalLn] = eebounds(Wp,Wn);

% Run EE with homotopy sequence in l
l=[0 10.^(linspace(-3,2,300))]; tic;[X,E,A]=ee(Wp,Wn,l,X0,[],[],100);toc

% Number of iterations at each lambda value and bounds
El=zeros(1,length(E)-1); for i=1:length(El) El(i)=length(E{i+1}); end;
figure(4); semilogx(l,El,'ro'); xlabel('\lambda'); ylabel('# its'); tmp = ylim;
hold on; plot([lb lb NaN ub ub],[0 tmp(2) NaN 0 tmp(2)],'bo-'); hold off;

% Result at each lambda
for i=1:length(l)
  MX = max(X{i+1}); mX = min(X{i+1});
  
  % 2D dataset coloured according to 1D embedding
  figure(1); scatter(Y(:,1),Y(:,2),12,X{i+1}); xlabel('y_1'); ylabel('y_2');
  title(['\lambda(' num2str(i) ')=' num2str(l(i))]);
  set(gca,'DataAspectRatio',[1 1 1]);
  
  % 1D embedding: x
  figure(2); plot(X{i+1},'bo-'); xlabel('n'); ylabel('x_n');
  title(['max(X)-min(X)=' num2str(MX-mX)]);
  
  % Error over iterations at this lambda value
  figure(3); plot(E{i+1},'ro-'); xlabel('it'); ylabel('E');
  
  pause;
end


% This shows the result and out-of-sample mappings for a given lambda
i=298;	% Pick a value of lambda

MX = max(X{i+1}); mX = min(X{i+1});
figure(5); scatter(Y(:,1),Y(:,2),12,X{i+1}); xlabel('Y_1'); ylabel('Y_2');
title(['\lambda(' num2str(i) ')=' num2str(l(i))]);
set(gca,'DataAspectRatio',[1 1 1],'Box','on');
figure(6); plot(X{i+1},'bo-'); xlabel('n'); ylabel('X_n');
title(['max(X)-min(X)=' num2str(MX-mX)]);

% ee_F
y = [Y;(Y(1:end-1,:)+Y(2:end,:))/2];
tic; [xN,eN,aN] = ee_F(y,Y,X{i+1},l(i),s,[],'N',1e-9,100); toc
figure(6); hold on; plot([1:N 1.5:1:N],xN,'k+'); hold off;

% ee_f
x = linspace(1.25*mX,1.25*MX,100)';
tic; [yN,eN,aN] = ee_f(x,Y,X{i+1},l(i),s,[],'N',1e-9,100); toc
figure(5); hold on; plot(yN(:,1),yN(:,2),'k+'); hold off;

