% [u,l,ulist,llist,Lp,evecLp,evalLp,Ln,evecLn,evalLn] = eebounds(Wp,Wn)
% Bounds on critical lambda for EE
%
% In:
%   Wp, Wn: NxN positive and negative weight matrices for EE.
% Out:
%   u, l: tightest upper and lower bound.
%   ulist, llist: all upper and lower bounds.
%   Lp, evecLp, evalLp: NxN graph Laplacian for the positive weight matrix,
%      its 1xN list of eigenvectors in ascending order, and its corresponding
%      N column eigenvectors.
%   Ln, evecLn, evalLn: ditto for the negative weight matrix.

% Copyright (c) 2009 by Miguel A. Carreira-Perpinan

function [u,l,ulist,llist,Lp,evecLp,evalLp,Ln,evecLn,evalLn] = eebounds(Wp,Wn)

N = size(Wp,1);
Lp = diag(sum(Wp,2)) - Wp; [evecLp,evalLp] = eig(full(Lp));
[evalLp,tmp] = sort(diag(evalLp)'); evecLp = evecLp(:,tmp);
Ln = diag(sum(Wn,2)) - Wn; [evecLn,evalLn] = eig(full(Ln));
[evalLn,tmp] = sort(diag(evalLn)'); evecLn = evecLn(:,tmp);
ulist = [evalLp(2:N)./evalLn(2:N) diag(Lp)'./diag(Ln)']; u = min(ulist);
z=find(Wn>0); llist = [evalLp(2)/evalLn(N) min(Wp(z)./Wn(z))]; l = max(llist);

